'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _es6Promise = require('es6-promise');

var _es6Promise2 = _interopRequireDefault(_es6Promise);

var _isImage = require('is-image');

var _isImage2 = _interopRequireDefault(_isImage);

var _isUrl = require('is-url');

var _isUrl2 = _interopRequireDefault(_isUrl);

var _slateDevLogger = require('slate-dev-logger');

var _slateDevLogger2 = _interopRequireDefault(_slateDevLogger);

var _mimeTypes = require('mime-types');

var _mimeTypes2 = _interopRequireDefault(_mimeTypes);

var _loadImageFile = require('./load-image-file');

var _loadImageFile2 = _interopRequireDefault(_loadImageFile);

var _path = require('path');

var _slateReact = require('slate-react');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * Insert images on drop or paste.
 *
 * @param {Object} options
 *   @property {Function} insertImage
 *   @property {Array} extensions (optional)
 * @return {Object} plugin
 */

function DropOrPasteImages() {
  var options = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};
  var insertImage = options.insertImage,
      extensions = options.extensions;


  if (options.applyTransform) {
    _slateDevLogger2.default.deprecate('0.6.0', 'The `applyTransform` argument to `slate-drop-or-paste-images` has been renamed to `insertImage` instead.');
    insertImage = options.applyTransform;
  }

  if (!insertImage) {
    throw new Error('You must supply an `insertImage` function.');
  }

  /**
   * Apply the change for a given file and update the editor with the result.
   *
   * @param {Change} change
   * @param {Editor} editor
   * @param {Blob} file
   * @return {Promise}
   */

  function asyncApplyChange(change, editor, file) {
    return _es6Promise2.default.resolve(insertImage(change, file)).then(function () {
      editor.onChange(change);
    });
  }

  /**
   * On drop or paste.
   *
   * @param {Event} event
   * @param {Change} change
   * @param {Editor} editor
   * @return {State}
   */

  function onInsert(event, change, editor) {
    var transfer = (0, _slateReact.getEventTranfser)(event);
    switch (transfer.type) {
      case 'files':
        return onInsertFiles(event, change, editor, transfer);
      case 'html':
        return onInsertHtml(event, change, editor, transfer);
      case 'text':
        return onInsertText(event, change, editor, transfer);
    }
  }

  /**
   * On drop or paste files.
   *
   * @param {Event} event
   * @param {Change} change
   * @param {Editor} editor
   * @param {Object} transfer
   * @return {Boolean}
   */

  function onInsertFiles(event, change, editor, transfer) {
    var target = transfer.target,
        files = transfer.files;
    var _iteratorNormalCompletion = true;
    var _didIteratorError = false;
    var _iteratorError = undefined;

    try {

      for (var _iterator = files[Symbol.iterator](), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
        var file = _step.value;

        if (extensions) {
          var ext = _mimeTypes2.default.extension(file.type);
          if (!extensions.includes(ext)) continue;
        }

        if (target) {
          change.select(target);
        }

        asyncApplyChange(change, editor, file);
      }
    } catch (err) {
      _didIteratorError = true;
      _iteratorError = err;
    } finally {
      try {
        if (!_iteratorNormalCompletion && _iterator.return) {
          _iterator.return();
        }
      } finally {
        if (_didIteratorError) {
          throw _iteratorError;
        }
      }
    }

    return true;
  }

  /**
   * On drop or paste html.
   *
   * @param {Event} event
   * @param {Change} change
   * @param {Editor} editor
   * @param {Object} transfer
   * @return {Boolean}
   */

  function onInsertHtml(event, change, editor, transfer) {
    var html = transfer.html,
        target = transfer.target;

    var parser = new DOMParser();
    var doc = parser.parseFromString(html, 'text/html');
    var body = doc.body;
    var firstChild = body.firstChild;
    if (firstChild.nodeName.toLowerCase() != 'img') return;

    var src = firstChild.src;

    if (extensions) {
      var ext = (0, _path.extname)(src).slice(1);
      if (!extensions.includes(ext)) return;
    }

    (0, _loadImageFile2.default)(src, function (err, file) {
      if (err) return;
      var c = editor.getState().change();
      if (target) c.select(target);
      asyncApplyChange(c, editor, file);
    });

    return true;
  }

  /**
   * On drop or paste text.
   *
   * @param {Event} event
   * @param {Change} change
   * @param {Editor} editor
   * @param {Object} transfer
   * @return {Boolean}
   */

  function onInsertText(event, change, editor, transfer) {
    var text = transfer.text,
        target = transfer.target;

    if (!(0, _isUrl2.default)(text)) return;
    if (!(0, _isImage2.default)(text)) return;

    (0, _loadImageFile2.default)(text, function (err, file) {
      if (err) return;
      var c = editor.getState().change();
      if (target) c.select(target);
      asyncApplyChange(c, editor, file);
    });

    return true;
  }

  /**
   * Return the plugin.
   *
   * @type {Object}
   */

  return {
    onDrop: onInsert,
    onPaste: onInsert
  };
}

/**
 * Export.
 *
 * @type {Function}
 */

exports.default = DropOrPasteImages;